# R script for Larsson, Tirado, & Wiens (2017)
# written by Stefan Wiens
# 31 march 2017

# use meta toolbox
library(meta)

# clear everything
rm(list=ls())

setwd("C:/Users/SWIENS/Documents/R Stefan/Metaanalysis/Maria Autism Metaanalysis")

# change to your own directory
sink("Meta analysis Odor and Autism results.txt", append=F, split=TRUE)

# In Excel, I selected the relevant data and copied/pasted them to Notepad++ to save as txt file
Dm=read.csv("data moderators.txt", sep="\t",header=TRUE, dec=",")

# Odor threshold task
D1=read.csv("data odor threshold.txt", sep="\t",header=TRUE, dec=",")

# Odor identification task
D2=read.csv("data odor identification.txt", sep="\t",header=TRUE, dec=",")

D = merge(Dm, D1, by = 'StudyID')

D = merge(D, D2, by = 'StudyID')

rm(Dm, D1, D2)

write.table(D, file = 'Data meta analysis.csv', sep = ";", row.names = F, col.names = T, na = '')

cor(D[,3:12], use = 'pairwise.complete.obs')

# meta
# odor threshold
# ===================
Dthr=D[is.na(D$Used.Hedges.g_thr)==FALSE, ]
# meta with standardized measures
mthr.gen <- metagen(Dthr$Used.Hedges.g_thr, sqrt(Dthr$Hedges.g.var_thr), sm="SMD",studlab=Dthr$Studyname.x)
# add number of subjects
mthr.gen$n.e = Dthr$nASD_thr
mthr.gen$n.c = Dthr$nCG_thr
cat('\nRESULTS Odor threshold\n-----------------------\n')
summary(mthr.gen)
mthr.gen$tau
#pdf(file="Forest odor threshold.pdf", paper = "a4r", width = 0, height = 0)
jpeg(file="1 Forest odor threshold.jpg", res = 300, width = 3000, height = 1400)

myxlab=paste("ASD better \U25C4 Hedges' g \U25BA CG better")
# getting the triangles in the figure labels was a total pain in the neck.

forest(mthr.gen,smlab = "Threshold task", xlab=myxlab, digits.se = 2,
       leftcols = c("studlab", "n.e", "n.c", "TE"), leftlabs = c("Study", "n-ASD", "n-CG", "Hedges' g"),
       rightcols = c("seTE", "ci", "w.random"), rightlabs = c("SEM", "95%-CI", "W(random)"),
       comb.fixed = F, prediction = T, col.predict = "black")
invisible(dev.off())

jpeg(file="2 Funnel odor threshold.jpg", res = 300, width = 3000, height = 1400)
funnel(mthr.gen, xlab=myxlab, 
       contour=0.95, col.contour = "lightgray",
       studlab = T, pos.studlab = 3, xlim = c(-1.2, 1.3))
title("Odor threshold task")
invisible(dev.off())

nowmodlbl = c('Percent Males in ASD','Mean Age in ASD','Mean IQ in ASD')

# Scatterplot Matrices from the car Package
library(car)
pdf(file="Scatterplot matrix odor threshold.pdf", paper = "a4r", width = 0, height = 0)
scatterplotMatrix(~Used.Hedges.g_thr+PercMaleASD+MeanAgeASDthre+MeanIQASDthre, data = Dthr, use = 'pairwise.complete.obs', smoother = F, 
                  main="Odor threshold task", var.labels = c('Odor Thre CG > ASD', nowmodlbl))
invisible(dev.off())

# fixed effect below (makes no sense that studies at the same
# level of the predictor should have the same effect)
#   mthr_pm.mr <- metareg(mthr.gen, Dthr$PercMaleASD, method.tau = "FE")
#   print(mthr_pm.mr, digits=2)

# Note: Meta regressions are run with DL (random effects)
# (It uses what it was fed with from the metagen thingy)
# meta regression 
cat(c('\nRESULTS Meta regression of Odor threshold: ', nowmodlbl[1], '\n-----------------------\n'))
mthr_pm.mr <- metareg(mthr.gen, Dthr$PercMaleASD)
print(mthr_pm.mr, digits=2)

# meta regression 
cat(c('\nRESULTS Meta regression of Odor threshold: ', nowmodlbl[2]), '\n-----------------------\n')
mthr_ma.mr <- metareg(mthr.gen, Dthr$MeanAgeASDthre)
print(mthr_ma.mr, digits=2)

# meta regression 
cat(c('\nRESULTS Meta regression of Odor threshold: ', nowmodlbl[3]), '\n-----------------------\n')
mthr_iq.mr <- metareg(mthr.gen, Dthr$MeanIQASDthre)
print(mthr_iq.mr, digits=2)


#pdf(file="Scatterplot odor threshold.pdf", paper = "a4r", width = 0, height = 0)
jpeg(file="5 Scatterplot odor threshold.jpg", res = 300, width = 3000, height = 2000)
par(mfrow=c(1,3)) 
# if you do not have raw data, the bubble function in meta will not work
# size by area
# r = sqrt(Area of circle / pi)
#
# Plot percent male in ASD
nowvalid = is.na( Dthr$PercMaleASD )==FALSE
circlesz = sqrt(mthr.gen$w.random[ nowvalid ]/pi)
# which studies?
mthr_pm.mr$.meta$x$studlab[mthr_pm.mr$not.na]; as.numeric(mthr_pm.mr$yi)
symbols(as.numeric(mthr_pm.mr$X[,2]), as.numeric(mthr_pm.mr$yi), circles= circlesz, inches = 0.2, 
        xlab = nowmodlbl[1], ylab = myxlab, main = nowmodlbl[1])
text(as.numeric(mthr_pm.mr$X[,2]), as.numeric(mthr_pm.mr$yi), 
     mthr_pm.mr$.meta$x$studlab[mthr_pm.mr$not.na], cex=1, pos=3, col="black", offset = 1.1)
abline(0,0, lty = 2)
abline(mthr_pm.mr$b[1], mthr_pm.mr$b[2])

# Plot mean age in ASD
nowvalid = is.na( Dthr$MeanAgeASDthre )==FALSE
circlesz = sqrt(mthr.gen$w.random[ nowvalid ]/pi)
# which studies?
mthr_ma.mr$.meta$x$studlab[mthr_ma.mr$not.na]; as.numeric(mthr_ma.mr$yi)
symbols(as.numeric(mthr_ma.mr$X[,2]), as.numeric(mthr_ma.mr$yi), circles= circlesz, inches = 0.2, 
        xlab = nowmodlbl[2] , ylab = myxlab, main = nowmodlbl[2])
text(as.numeric(mthr_ma.mr$X[,2]), as.numeric(mthr_ma.mr$yi), 
     mthr_ma.mr$.meta$x$studlab[mthr_ma.mr$not.na], cex=1, pos=3, col="black", offset = 1.1)
abline(0,0, lty = 2)
abline(mthr_ma.mr$b[1], mthr_ma.mr$b[2])

# Plot mean IQ in ASD threshold
nowvalid = is.na( Dthr$MeanIQASDthre )==FALSE
circlesz = sqrt(mthr.gen$w.random[ nowvalid ]/pi)
# which studies?
mthr_iq.mr$.meta$x$studlab[mthr_iq.mr$not.na]; as.numeric(mthr_iq.mr$yi)
symbols(as.numeric(mthr_iq.mr$X[,2]), as.numeric(mthr_iq.mr$yi), circles= circlesz, inches = 0.2, 
        xlab = nowmodlbl[3] , ylab = myxlab, main = nowmodlbl[3])
text(as.numeric(mthr_iq.mr$X[,2]), as.numeric(mthr_iq.mr$yi),
     mthr_iq.mr$.meta$x$studlab[mthr_iq.mr$not.na], cex=1, pos=3, col="black", offset = 1.1)
abline(0,0, lty = 2)
abline(mthr_iq.mr$b[1], mthr_iq.mr$b[2])
invisible(dev.off())

# Note that age and IQ correlate (r > .72), so it is difficult to isolate contributions from these two variables
cor(D[,c(4,6,10)], use = 'pairwise.complete.obs')

# meta
# odor identification
# ===================
Did=D[is.na(D$Used.Hedges.g_id)==FALSE, ]
# meta with standardized measures
mid.gen <- metagen(Did$Used.Hedges.g_id, sqrt(Did$Hedges.g.var_id), sm="SMD",studlab=Did$Studyname.x)
# add number of subjects
mid.gen$n.e = Did$nASD_id
mid.gen$n.c = Did$nCG_id
cat('\nRESULTS Odor identification\n-----------------------\n')
summary(mid.gen)
mid.gen$tau

jpeg(file="3 Forest odor identification.jpg", res = 300, width = 3000, height = 1400)
forest(mid.gen,smlab = "Identification task", xlab=myxlab, digits.se = 2,
       leftcols = c("studlab", "n.e", "n.c", "TE"), leftlabs = c("Study", "n-ASD", "n-CG", "Hedges' g"),
       rightcols = c("seTE", "ci", "w.random"), rightlabs = c("SEM", "95%-CI", "W(random)"),
       comb.fixed = F, prediction = T, col.predict = "black")
# The code below plots both fixed and random
# forest(mid.gen,smlab = "Identification task", xlab=myxlab, digits.se = 2,
#        leftcols = c("studlab", "n.e", "n.c", "TE"), leftlabs = c("Study", "n-ASD", "n-CG", "Hedges' g"),
#        rightcols = c("seTE", "ci", "w.fixed", "w.random"), rightlabs = c("SEM", "95%-CI", "W(fixed)", "W(random)"))
invisible(dev.off())

cat('\nSensitivity analysis (without Suzuki 2003): RESULTS Odor identification\n----------------\n')
# Sensitivity analysis
# Exclude Suzuki 03 (g = 1.99)
Did2 = Did[Did$StudyID!=1,]
# meta with standardized measures
mid2.gen <- metagen(Did2$Used.Hedges.g_id, sqrt(Did2$Hedges.g.var_id), sm="SMD",studlab=Did2$Studyname.x)
# add number of subjects
mid2.gen$n.e = Did2$nASD_id
mid2.gen$n.c = Did2$nCG_id
summary(mid2.gen)
mid2.gen$tau
forest(mid2.gen,smlab = "Identification task", xlab=myxlab, digits.se = 2,
       leftcols = c("studlab", "n.e", "n.c", "TE"), leftlabs = c("Study", "n-ASD", "n-CG", "Hedges' g"),
       rightcols = c("seTE", "ci", "w.random"), rightlabs = c("SEM", "95%-CI", "W(random)"),
       comb.fixed = F, prediction = T)
# end of sensitivity analysis
# go back to all studies

jpeg(file="4 Funnel odor identification.jpg", res = 300, width = 3000, height = 1400)
# Addo and Brewer overlap: shift Addo (#9) downwards
tmplbl = mid.gen$studlab
tmplbl[8] = ""
funnel(mid.gen, xlab=myxlab,
       contour=0.95, col.contour = "lightgray",
       studlab = tmplbl, pos.studlab = 3, xlim = c(-0.8, 2.1))
text(-0.57,0.36, mid.gen$studlab[8], pos = 1, cex = 0.8)
title("Odor identification task")
invisible(dev.off())

nowmodlbl = c('Percent Males in ASD','Mean Age in ASD','Mean IQ in ASD')

# Scatterplot Matrices from the car Package
library(car)
pdf(file="Scatterplot matrix odor identification.pdf", paper = "a4r", width = 0, height = 0)
scatterplotMatrix(~Used.Hedges.g_id+PercMaleASD+MeanAgeASDiden+MeanIQASDiden, data = Did, use = 'pairwise.complete.obs', smoother = F, 
                  main="Odor identification task", var.labels = c('Odor Id CG > ASD', nowmodlbl))
invisible(dev.off())

# meta regression 
cat(c('\nRESULTS Meta regression of Odor identification: ', nowmodlbl[1], '\n-----------------------\n'))
mid_pm.mr <- metareg(mid.gen, Did$PercMaleASD)
print(mid_pm.mr, digits=2)

# meta regression 
cat(c('\nRESULTS Meta regression of Odor identification: ', nowmodlbl[2]), '\n-----------------------\n')
mid_ma.mr <- metareg(mid.gen, Did$MeanAgeASDiden)
print(mid_ma.mr, digits=2)

# meta regression 
cat(c('\nRESULTS Meta regression of Odor identification: ', nowmodlbl[3]), '\n-----------------------\n')
mid_iq.mr <- metareg(mid.gen, Did$MeanIQASDiden)
print(mid_iq.mr, digits=2)

#pdf(file="Scatterplot odor identification.pdf", paper = "a4r", width = 0, height = 0)
jpeg(file="6 Scatterplot odor identification.jpg", res = 300, width = 3000, height = 2000)
par(mfrow=c(1,3)) 
# if you do not have raw data, the bubble function in meta will not work
# size by area
# r = sqrt(Area of circle / pi)
#
# Plot percent male in ASD
nowvalid = is.na( Did$PercMaleASD )==FALSE
circlesz = sqrt(mid.gen$w.random[ nowvalid ]/pi)
# which studies?
mid_pm.mr$.meta$x$studlab[mid_pm.mr$not.na]; as.numeric(mid_pm.mr$yi)
symbols(as.numeric(mid_pm.mr$X[,2]), as.numeric(mid_pm.mr$yi), circles= circlesz, inches = 0.2, 
        xlab = nowmodlbl[1], ylab = myxlab, main = nowmodlbl[1])
text(as.numeric(mid_pm.mr$X[,2]), as.numeric(mid_pm.mr$yi), 
     mid_pm.mr$.meta$x$studlab[mid_pm.mr$not.na], cex=1, pos=3, col="black", offset = 1.1)
abline(0,0, lty = 2)
abline(mid_pm.mr$b[1], mid_pm.mr$b[2])

# Plot mean age in ASD
nowvalid = is.na( Did$MeanAgeASDiden )==FALSE
circlesz = sqrt(mid.gen$w.random[ nowvalid ]/pi)
# which studies?
mid_ma.mr$.meta$x$studlab[mid_ma.mr$not.na]; as.numeric(mid_ma.mr$yi)
symbols(as.numeric(mid_ma.mr$X[,2]), as.numeric(mid_ma.mr$yi), circles= circlesz, inches = 0.2, 
        xlab = nowmodlbl[2] , ylab = myxlab, main = nowmodlbl[2])
text(as.numeric(mid_ma.mr$X[,2]), as.numeric(mid_ma.mr$yi),
     mid_ma.mr$.meta$x$studlab[mid_ma.mr$not.na], cex=1, pos=3, col="black", offset = 1.1)
abline(0,0, lty = 2)
abline(mid_ma.mr$b[1], mid_ma.mr$b[2])

# Plot mean IQ in ASD identification
nowvalid = is.na( Did$MeanIQASDiden )==FALSE
circlesz = sqrt(mid.gen$w.random[ nowvalid ]/pi)
# which studies?
mid_iq.mr$.meta$x$studlab[mid_iq.mr$not.na]; as.numeric(mid_iq.mr$yi)
symbols(as.numeric(mid_iq.mr$X[,2]), as.numeric(mid_iq.mr$yi), circles= circlesz, inches = 0.2, 
        xlab = nowmodlbl[3] , ylab = myxlab, main = nowmodlbl[3])
text(as.numeric(mid_iq.mr$X[,2]), as.numeric(mid_iq.mr$yi),
     mid_iq.mr$.meta$x$studlab[mid_iq.mr$not.na], cex=1, pos=3, col="black", offset = 1.1)
abline(0,0, lty = 2)
abline(mid_iq.mr$b[1], mid_iq.mr$b[2])
invisible(dev.off())

# Note that variables correlate (r > .50), so it is difficult to isolate contributions from different variables
cor(D[,c(4,8,12)], use = 'pairwise.complete.obs')

# turn off saving output to file
sink()
save.image("~/R Stefan/Metaanalysis/Maria Autism Metaanalysis/Rspace.RData")